{-# LANGUAGE ScopedTypeVariables #-}

module Test.API.Export (tests) where

import Data.ByteString.Lazy qualified as BSL
import Data.Version (showVersion)
import Database.SQLite.Simple qualified as S
-- import Fmt (build)
import Gargantext.API.Node.Corpus.Export.Types (CorpusSQLite(..))
import Gargantext.API.Node.Corpus.Export.Utils (withTempSQLiteDir)
import Gargantext.Core.Types (unNodeId)
import Gargantext.Core.Types.Individu
import Gargantext.Prelude hiding (get)
import Paths_gargantext qualified as PG -- cabal magic build module
import Servant.API.ResponseHeaders (Headers(getResponse))
import Servant.Auth.Client ()
import Servant.Client
import Test.API.Prelude (checkEither)
import Test.API.Routes (get_corpus_sqlite_export)
import Test.API.Setup (withTestDBAndPort, dbEnvSetup, SpecContext (..))
import Test.API.UpdateList (createFortranDocsList)
import Test.Hspec
import Test.Hspec.Wai.Internal (withApplication)
import Test.Utils (withValidLogin)


tests :: Spec
tests = sequential $ aroundAll withTestDBAndPort $ beforeAllWith dbEnvSetup $ do
  describe "Export API" $ do
    describe "GET /api/v1.0/corpus/cId/sqlite" $ do
      it "returns correct SQLite db" $ \ctx -> do
        let port = _sctx_port ctx
        withApplication (_sctx_app ctx) $ do
          withValidLogin port "alice" (GargPassword "alice") $ \clientEnv token -> do
            cId <- createFortranDocsList (_sctx_env ctx) port clientEnv token
            void $ liftIO $ do
              (CorpusSQLite { _cs_bs }) <-
                (checkEither $ runClientM (get_corpus_sqlite_export token cId) clientEnv) >>= (pure . getResponse)
              withTempSQLiteDir $ \(_fp, _fname, fpath) -> do
                BSL.writeFile fpath _cs_bs
                S.withConnection fpath $ \conn -> do
                  [S.Only cId'] <- S.query_ conn "SELECT value FROM info WHERE key = 'corpusId'"
                  cId' `shouldBe` unNodeId cId
                  -- [S.Only lId'] <- S.query_ conn "SELECT value FROM info WHERE key = 'listId'"
                  -- lId' `shouldBe` unNodeId listId
                  [S.Only version] <- S.query_ conn "SELECT value FROM info WHERE key = 'gargVersion'"
                  version `shouldBe` showVersion PG.version
                  [S.Only corpoLen] <- S.query conn "SELECT COUNT(*) FROM corpus WHERE id = ?" (S.Only $ unNodeId cId)
                  corpoLen `shouldBe` (1 :: Int)
                  -- [S.Only listLen] <- S.query conn "SELECT COUNT(*) FROM lists WHERE id = ?" (S.Only $ unNodeId listId)
                  -- listLen `shouldBe` (1 :: Int)

                  [S.Only ngramsLen] <- S.query_ conn "SELECT COUNT(*) FROM ngrams"
                  ngramsLen `shouldBe` (0 :: Int)
                  [S.Only docsLen] <- S.query_ conn "SELECT COUNT(*) FROM documents"
                  docsLen `shouldBe` (2 :: Int)
